# ---------------------------------------------------------------------------
# PURPOSE
# Small tutorial demonstrating how to use arrays in MIPS assembly.
#
# Concepts introduced:
#   - Array of integers.
#   - Array of strings.
#   - Loading an elment of an array to a register.
#   - Looping over all elements in an array.
#
# AUTHOR Karl Marklund <karl.marklund@it.uu.se> 2015-12-10, First version.
# modified by <Henrik.Schulze.0353@student.uu.se> 2017-04-04.
# ---------------------------------------------------------------------------

# DATA SEGMENT
# Data used by the program is declared in the data segment.
# ---------------------------------------------------------------------------
	.data
iarr:	.word 10, 11, 12	# Store three consequtive 4 byte words in memory (12 bytes).
one:	.asciiz "One\n"		# Three strings. Each label is an address (pointer) to a null-terminated string.
two: 	.asciiz "Two\n"
three:	.asciiz "Three\n"
# In sarr each value is a label that gets replaced by the label address. Thus, sarr is an array of pointers to strings, an array of strings:
sarr:	.word one, two, three

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# MAIN
# Text segment (code) for the program.
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	.text
main:	# Execution starts at the label main.
li $v0, 4	# li - Load Immediate - load system call code 4 (print string) into $v0:  addiu $v0, $zero, 4.
la $s1, sarr	# Load Address of 1st element of string array: lui $at,sarr  &  sw $s1,Off($at); Off=sarr-iarr

lw $a0, 0($s1)	# NOTE! Load into $a0 the memory address that 0($s1) = 0x10010020 POINTS to: 0x1001000c. (!)
syscall		# Print the 1st element, "One\n" at address 0x10010000 + 0x0000000c =  0x1001000c.

lw $a0, 4($s1)	# Print the 2nd element, offset 4: 4(0x10010020) = 0x10010024 POINTS to 0x10010011.
syscall		# Print the 2nd element, "Two\n" at address 0x1001000c + 0x00000005 =  0x10010011.

#lw $a0, 8($s1)	# Print 3rd element (offset 8).
lw $a0, 8($s1)	# Print the 3rd element, offset 8: 8(0x10010020) = 0x10010028 POINTS to 0x10010016.
syscall		# Print the 3rd element, "Three\n" at address 0x10010011 + 0x00000005 =  0x10010016.

##### Loop over all strings in array of strings
la $t0, sarr	# Load Address of the array POINTER into $t0.
li $t1, 0	# Initialize array offset.
li $t2, 8	# Last offset.
li $v0, 4	# System call code 4 (print_string).
loop:
lw $a0,sarr($t1)# NOTE! Load into $a0 the memory address that sarr($t1) = 0x10010020 POINTS to: 0x1001000c. (!)
syscall		# Print the string that starts at 0x1001000c, 0x10010011, 0x10010016.
addi $t1,$t1,4	# Next element in the loop, i.e., increment offset by 4.
# Done? Or loop once more?
ble $t1,$t2,loop# ble - Branch if $t1 is Less or Equal to $t2: slt $at,$t2,$t1  &  beq $at,$zero,-5. (offset=-5)

li $v0, 10   	# System call code 10 (exit).
syscall		# exit.
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# The code below never runs:
la $s0, iarr	# Load Address of the 1st element in the array:  lui $at, iarr  &  ori $s0, $at, 0 (offset 0).
lw $t0, 0($s0)	# Load the 1st element of the integer array into $t0 (offset 0).
lw $t1, 4($s0)	# Load the 2nd element of the integer array into $t1 (offset 4 bytes = 1 word).
lw $t2, 8($s0)	# Load the 3rd element of the integer array into $t2 (offset 8 bytes = 2 words).
