# ---------------------------------------------------------------------------
# PURPOSE
# Small MIPS assembly tutorial demonstrating unconditional and conditional jumps.
#   - An unconditional jump always occurs. There is no condition to check.
#   - A conditional jump is called a branch in MIPS.
#
# The tutorial introduces the following MIPS instructions.
#   Instruction   Meaning
#   ----------------------------------------------
#   j             Unconditional jump
#   blt           Branch if Less Than
#   bge           Branch if Greater or Equal
#
# In this tutorial the following control structures will be implemented using conditional jumps:
#   - if-then-else
#   - infinite loop
#
# AUTHOR Karl Marklund <karl.marklund@it.uu.se> 2015-12-10, First version.
# modified by <Henrik.Schulze.0353@student.uu.se> 2017-04-04.
# ---------------------------------------------------------------------------

# DATA SEGMENT
# Data used by the program is declared in the data segment.
# ---------------------------------------------------------------------------
	.data
yes:	.asciiz "Yes ($t0 < $t1)\n"
no:	.asciiz "No ($t0 >= $t1)\n"

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# MAIN
# Text segment (code) for the program.
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	.text
main:	# Execution starts at the label main.
addi $s0, $zero, 0 	# Initialize $s0 = 0. (Just to be sure?)
j increm_s1	# j unconditionally Jumps to an address which may be given by a label (here increm_s1).

increm_s0:
addi $s0, $s0, 1	# This code will never be reached.

increm_s1:
addi $s1, $s1, 1

##### if-then-else, example 1
li $t0, 1 	# A
li $t1, 9	# B
# Pseudo-code: IF A < B PRINT "Yes" ELSE PRINT "No". Now, lets's translate this pseudo-code to assembly.
# First we need to test if $t0 < $t1 and make a decision based on the result. For this purpose we can use blt, Branch if Less Than:
blt $t0,$t1,lt	# blt - Branch to lt if $t0 Less Than $t1. Else continue. slt d,r,s - Set d=1 if r Less Than s.

not_lt:
la $a0, no	# else clause: $t0 >= $t1. Load the address of the no string to print "No ($t0 >= $t1)\n".
j print_lt	# Unconditionally jump to the print_lt label.

lt:
la $a0, yes	# if clause: $t0 < $t1. Load address of the string to print "Yes ($t0 < $t1)\n". Continue.

print_lt:
li $v0, 4	# System call code 4 (print_string).
syscall		# Print the string in $a0.

##### if-then-else, example 2
# An alternative is to use the bge (Branch if Greater or Equal) instruction.
bge $t0,$t1,ge	# bge - Branch to ge if $t0 Greater or Equal to $t1. Otherwise next instruct. (slt as on line 49)

not_ge:
la $a0, yes	# case $t0 < $t1. Load the address of the yes string to print "Yes ($t0 < $t1)\n".
j print_ge	# Unconditionally jump to the label print_ge. (Line 74.)

ge:
la $a0, no	# case $t0 >= $t1. Load the address of the no string to print "No ($t0 >= $t1)\n". Continue.

print_ge:
li $v0, 4	# System call code 4 (print_string).
syscall		# Print the string that $a0 POINTS to.

##### Infinte loop, here we use the j instruction to construct an infinite loop:
forever:
addi $s0,$s0,1	# Increment counter by 1.
# When you single step, observe the highlighted instruction in the text segment.
# Note that the register $s0 is highlighted in the registers pane as the value of $s0 is incremented.
j forever	# Unconditionally jump to the label forever. (Line 79.)
# This code will never be reached.
